@@ -5,6 +5,7 @@ import (
 	"strconv"
 	"strings"
 	"time"
+	"net"
 	"regexp"
 	"google.golang.org/grpc"
 )
@@ -17,12 +18,23 @@ type grpcTransport struct {
 }
 
 type GrpcTransport interface {
+	// SetLocation set client connection to the given grpc target
 	SetLocation(value string) GrpcTransport
+	// Location get grpc target
 	Location() string
+	// SetRequestTimeout set timeout in grpc request
 	SetRequestTimeout(value time.Duration) GrpcTransport
+	// RequestTimeout get timeout in grpc request
 	RequestTimeout() time.Duration
+	// SetDialTimeout set timeout in grpc dial
 	SetDialTimeout(value time.Duration) GrpcTransport
+	// DialTimeout get timeout in grpc dial
 	DialTimeout() time.Duration
+	// SetClientConnection set grpc DialContext
+	// SetClientConnection and (SetLocation, SetDialTimeout) are mutually exclusive
+	SetClientConnection(con *grpc.ClientConn) GrpcTransport
+	// ClientConnection get grpc DialContext
+	ClientConnection() *grpc.ClientConn
 }
 
 // Location
@@ -55,9 +67,19 @@ func (obj *grpcTransport) SetDialTimeout(value time.Duration) GrpcTransport {
 	return obj
 }
 
+func (obj *grpcTransport) ClientConnection() *grpc.ClientConn {
+	return obj.clientConnection
+}
+
+func (obj *grpcTransport) SetClientConnection(con *grpc.ClientConn) GrpcTransport {
+	obj.clientConnection = con
+	return obj
+}
+
 type httpTransport struct {
 	location string
 	verify   bool
+	conn     net.Conn
 }
 
 type HttpTransport interface {
@@ -90,8 +112,9 @@ func (obj *httpTransport) SetVerify(value bool) HttpTransport {
 }
 
 type api struct {
-	grpc *grpcTransport
-	http *httpTransport
+	grpc     *grpcTransport
+	http     *httpTransport
+	warnings string
 }
 
 type Api interface {
@@ -100,6 +123,11 @@ type Api interface {
 	NewHttpTransport() HttpTransport
 	hasHttpTransport() bool
 	Close() error
+	// Warnings Api is only for testing purpose
+	// and not intended to use in production
+	Warnings() string
+	deprecated(message string)
+	under_review(message string)
 }
 
 // NewGrpcTransport sets the underlying transport of the Api as grpc
@@ -137,6 +165,20 @@ func (api *api) hasHttpTransport() bool {
 	return api.http != nil
 }
 
+func (api *api) Warnings() string {
+	return api.warnings
+}
+
+func (api *api) deprecated(message string) {
+	api.warnings = message
+	fmt.Printf("warning: %s\n", message)
+}
+
+func (api *api) under_review(message string) {
+	api.warnings = message
+	fmt.Printf("warning: %s\n", message)
+}
+
 // HttpRequestDoer will return True for HTTP transport
 type httpRequestDoer interface {
 	Do(req *http.Request) (*http.Response, error)
@@ -149,19 +191,54 @@ type httpClient struct {
 
 // All methods that perform validation will add errors here
 // All api rpcs MUST call Validate
-var validation []string
+type Constraints interface {
+	ValueOf(name string) interface{}
+}
 
-func validationResult() error {
-	if len(validation) > 0 {
-		validation = append(validation, "validation errors")
-		errors := strings.Join(validation, "\n")
-		validation = nil
+type validation struct {
+	validationErrors []string
+	warnings         []string
+	constraints      map[string]map[string]Constraints
+}
+
+type Validation interface {
+	validationResult() error
+	deprecated(message string)
+	under_review(message string)
+	Warnings() []string
+}
+
+func (obj *validation) validationResult() error {
+	obj.constraints = make(map[string]map[string]Constraints)
+	if len(obj.validationErrors) > 0 {
+		obj.validationErrors = append(obj.validationErrors, "validation errors")
+		errors := strings.Join(obj.validationErrors, "\n")
+		obj.validationErrors = nil
 		return fmt.Errorf(errors)
 	}
 	return nil
 }
 
-func validateMac(mac string) error {
+func (obj *validation) Warnings() []string {
+	if len(obj.warnings) > 0 {
+		warns := obj.warnings
+		obj.warnings = nil
+		return warns
+	}
+	return obj.warnings
+}
+
+func (obj *validation) deprecated(message string) {
+	fmt.Printf("warning: %s\n", message)
+	obj.warnings = append(obj.warnings, message)
+}
+
+func (obj *validation) under_review(message string) {
+	fmt.Printf("warning: %s\n", message)
+	obj.warnings = append(obj.warnings, message)
+}
+
+func (obj *validation) validateMac(mac string) error {
 	macSlice := strings.Split(mac, ":")
 	if len(macSlice) != 6 {
 		return fmt.Errorf(fmt.Sprintf("Invalid Mac address %s", mac))
@@ -176,7 +253,7 @@ func validateMac(mac string) error {
 	return nil
 }
 
-func validateIpv4(ip string) error {
+func (obj *validation) validateIpv4(ip string) error {
 	ipSlice := strings.Split(ip, ".")
 	if len(ipSlice) != 4 {
 		return fmt.Errorf(fmt.Sprintf("Invalid Ipv4 address %s", ip))
@@ -191,7 +268,7 @@ func validateIpv4(ip string) error {
 	return nil
 }
 
-func validateIpv6(ip string) error {
+func (obj *validation) validateIpv6(ip string) error {
 	ip = strings.Trim(ip, " \t")
 	if strings.Count(ip, " ") > 0 || strings.Count(ip, ":") > 7 ||
 		strings.Count(ip, "::") > 1 || strings.Count(ip, ":::") > 0 ||
@@ -231,7 +308,7 @@ func validateIpv6(ip string) error {
 	return nil
 }
 
-func validateHex(hex string) error {
+func (obj *validation) validateHex(hex string) error {
 	matched, err := regexp.MatchString(`^[0-9a-fA-F]+$|^0[x|X][0-9a-fA-F]+$`, hex)
 	if err != nil || !matched {
 		return fmt.Errorf(fmt.Sprintf("Invalid hex value %s", hex))
@@ -239,19 +316,19 @@ func validateHex(hex string) error {
 	return nil
 }
 
-func validateSlice(valSlice []string, sliceType string) error {
+func (obj *validation) validateSlice(valSlice []string, sliceType string) error {
 	indices := []string{}
 	var err error
 	for i, val := range valSlice {
 		if sliceType == "mac" {
-			err = validateMac(val)
+			err = obj.validateMac(val)
 		} else if sliceType == "ipv4" {
-			err = validateIpv4(val)
+			err = obj.validateIpv4(val)
 
 		} else if sliceType == "ipv6" {
-			err = validateIpv6(val)
+			err = obj.validateIpv6(val)
 		} else if sliceType == "hex" {
-			err = validateHex(val)
+			err = obj.validateHex(val)
 		} else {
 			return fmt.Errorf(fmt.Sprintf("Invalid slice type received <%s>", sliceType))
 		}
@@ -268,18 +345,76 @@ func validateSlice(valSlice []string, sliceType string) error {
 	return nil
 }
 
-func validateMacSlice(mac []string) error {
-	return validateSlice(mac, "mac")
+func (obj *validation) validateMacSlice(mac []string) error {
+	return obj.validateSlice(mac, "mac")
 }
 
-func validateIpv4Slice(ip []string) error {
-	return validateSlice(ip, "ipv4")
+func (obj *validation) validateIpv4Slice(ip []string) error {
+	return obj.validateSlice(ip, "ipv4")
 }
 
-func validateIpv6Slice(ip []string) error {
-	return validateSlice(ip, "ipv6")
+func (obj *validation) validateIpv6Slice(ip []string) error {
+	return obj.validateSlice(ip, "ipv6")
 }
 
-func validateHexSlice(hex []string) error {
-	return validateSlice(hex, "hex")
+func (obj *validation) validateHexSlice(hex []string) error {
+	return obj.validateSlice(hex, "hex")
 }
+
+// TODO: restore behavior
+// func (obj *validation) createMap(objName string) {
+// 	if obj.constraints == nil {
+// 		obj.constraints = make(map[string]map[string]Constraints)
+// 	}
+// 	_, ok := obj.constraints[objName]
+// 	if !ok {
+// 		obj.constraints[objName] = make(map[string]Constraints)
+// 	}
+// }
+
+// TODO: restore behavior
+// func (obj *validation) isUnique(objectName, value string, object Constraints) bool {
+// 	if value == "" {
+// 		return true
+// 	}
+
+// 	obj.createMap("globals")
+// 	_, ok := obj.constraints["globals"][value]
+// 	unique := false
+// 	if !ok {
+// 		obj.constraints["globals"][value] = object
+// 		obj.createMap(objectName)
+// 		obj.constraints[objectName][value] = object
+// 		unique = true
+// 	}
+// 	return unique
+// }
+
+// TODO: restore behavior
+// func (obj *validation) validateConstraint(objectName []string, value string) bool {
+// 	if value == "" {
+// 		return false
+// 	}
+// 	found := false
+// 	for _, object := range objectName {
+// 		obj_ := strings.Split(object, ".")
+// 		strukt, ok := obj.constraints[obj_[0]]
+// 		if !ok {
+// 			continue
+// 		}
+// 		for _, object := range strukt {
+// 			intf := object.ValueOf(obj_[1])
+// 			if intf == nil {
+// 				continue
+// 			}
+// 			if value == fmt.Sprintf("%v", intf) {
+// 				found = true
+// 				break
+// 			}
+// 		}
+// 		if found {
+// 			break
+// 		}
+// 	}
+// 	return found
+// }
