@@ -1,16 +1,15 @@
 from flask import Flask, request, Response
 import threading
 import json
-import time
 import os
 import importlib
 import sys
-import socket
 
 
 app = Flask(__name__)
 app.CONFIG = None
 app.PACKAGE = None
+app.UPDATE_CONFIG = None
 app.PORT = 18080
 app.HOST = "0.0.0.0"
 
@@ -31,6 +30,14 @@ def set_config():
         return Response(status=200)
 
 
+@app.route("/api/config", methods=["PATCH"])
+def update_configuration():
+    config = app.PACKAGE.Api().update_config()
+    config.deserialize(request.data.decode("utf-8"))
+    app.UPDATE_CONFIG = config
+    return Response(status=200)
+
+
 @app.route("/api/config", methods=["GET"])
 def get_config():
     app.CONFIG.a = "asdf"
@@ -70,4 +77,3 @@ class OpenApiServer(object):
         self._web_server_thread.setDaemon(True)
         self._web_server_thread.start()
         return self
-
